// naumann@stce.rwth-aachen.de

#include <limits>
#include <cmath>

template<typename T>
T f1(const T &x) { return sin(x); }

template<typename T>
T f2(const T &x) { return cos(x); }

template<typename T, typename PT>
void f(T &x, const PT &p, const PT &w) {
  T a=f1(x);
  T b=f2(x);
  x=1/(1+exp(-(x-p)/w));
  x=a*(1-x)+b*x;
}

template<typename T, typename PT>
void first_derivative(T &x, const PT &p, const PT &w, T &dx) {
  T h=(x==0) ? sqrt(std::numeric_limits<T>::epsilon())
             : sqrt(std::numeric_limits<T>::epsilon())*fabs(x);
  T xph=x+h; f(xph,p,w);
  T xmh=x-h; f(xmh,p,w);
  dx=(xph-xmh)/(2*h);
  f(x,p,w);
}

#include <iostream>
#include <cassert>

int main(int argc, char *argv[]) {
  assert(argc==4);
  using namespace std;
  double x=stof(argv[1]), dx;
  double p=stof(argv[2]), w=stof(argv[3]); assert(w>0);
  first_derivative(x,p,w,dx);
  cout << x << " " << dx << endl;
  return 0;
}


