// naumann@stce.rwth-aachen.de

#include <limits>
#include <cmath>

template<typename T>
T f1(const T &x) { return x-1; }

template<typename T>
T f2(const T &x) { return x+1; }

template<typename T, typename PT>
T f(T x, const PT &p, const PT &w) {
  T a=f1(x);
  T b=f2(x);
  T c=1/(1+exp(-(x-p)/w));
  return a*(1-c)+b*c;
}

template<typename T, typename PT>
T first_derivative(T &x, const PT &p, const PT &w) {
  T h=(x==0) ? sqrt(std::numeric_limits<T>::epsilon())
             : sqrt(std::numeric_limits<T>::epsilon())*fabs(x);
  T xph=x+h; xph=f(xph,p,w);
  T xmh=x-h; xmh=f(xmh,p,w);
  return (xph-xmh)/(2*h);
}

#include <iostream>
#include <cassert>

template<typename T, typename PT>
void newton(T& x, const PT& p, const PT& w, const PT& eps) {
  using namespace std;
  do {
    x=x-f(x,p,w)/first_derivative(x,p,w);
    cout << "x=" << x << "; "
	 << "f(x)=" << f(x,p,w) << "; "
	 << "f'(x)=" << first_derivative(x,p,w) << endl;
  } while (fabs(f(x,p,w))>eps);
}

int main(int argc, char *argv[]) {
  assert(argc==5);
  using namespace std;
  double x=stof(argv[1]);
  double p=stof(argv[2]), w=stof(argv[3]), eps=stof(argv[4]); assert(w>0);
  newton(x,p,w,eps);
  return 0;
}


